@@ -102,6 +102,7 @@ class TestConfig(base.TestCase):
         self.assertEqual('123456789', cc.auth['project_domain_id'])
         self.assertNotIn('domain_id', cc.auth)
         self.assertNotIn('domain-id', cc.auth)
+        self.assertNotIn('domain_id', cc)
 
     def test_get_one_cloud_domain_scoped(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -118,6 +119,7 @@ class TestConfig(base.TestCase):
         self.assertEqual('awesome-domain', cc.auth['user_domain_id'])
         self.assertEqual('awesome-domain', cc.auth['project_domain_id'])
         self.assertNotIn('domain_id', cc.auth)
+        self.assertNotIn('domain_id', cc)
 
     def test_get_one_cloud_with_hyphenated_project_id(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -192,11 +194,19 @@ class TestConfig(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
         cc = c.get_one_cloud('_test-cloud-networks_')
         self.assertEqual(
-            ['a-public', 'another-public'], cc.get_external_networks())
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_networks())
         self.assertEqual(
-            ['a-private', 'another-private'], cc.get_internal_networks())
+            ['a-private', 'another-private', 'split-no-default'],
+            cc.get_internal_networks())
         self.assertEqual('another-private', cc.get_nat_destination())
         self.assertEqual('another-public', cc.get_default_network())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-no-default'],
+            cc.get_external_ipv4_networks())
+        self.assertEqual(
+            ['a-public', 'another-public', 'split-default'],
+            cc.get_external_ipv6_networks())
 
     def test_get_one_cloud_no_networks(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -402,6 +412,42 @@ class TestConfigArgparse(base.TestCase):
         cc = c.get_one_cloud(
             argparse=options, **kwargs)
         self.assertEqual(cc.region_name, 'region2')
+        self.assertEqual(cc.auth['password'], 'authpass')
+        self.assertEqual(cc.snack_type, 'cookie')
+
+    def test_get_one_cloud_precedence_osc(self):
+        c = config.OpenStackConfig(
+            config_files=[self.cloud_yaml],
+            vendor_files=[self.vendor_yaml],
+        )
+
+        kwargs = {
+            'auth': {
+                'username': 'testuser',
+                'password': 'authpass',
+                'project-id': 'testproject',
+                'auth_url': 'http://example.com/v2',
+            },
+            'region_name': 'kwarg_region',
+            'password': 'ansible_password',
+            'arbitrary': 'value',
+        }
+
+        args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            password='argpass',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+        )
+
+        options = argparse.Namespace(**args)
+        cc = c.get_one_cloud_osc(
+            argparse=options,
+            **kwargs
+        )
+        self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.auth['password'], 'argpass')
         self.assertEqual(cc.snack_type, 'cookie')
 
@@ -873,7 +919,9 @@ class TestBackwardsCompatibility(base.TestCase):
         expected = {
             'networks': [
                 {'name': 'private', 'routes_externally': False,
-                 'nat_destination': False, 'default_interface': False},
+                 'nat_destination': False, 'default_interface': False,
+                 'routes_ipv4_externally': False,
+                 'routes_ipv6_externally': False},
             ]
         }
         self.assertEqual(expected, result)
